@@ -51,14 +51,53 @@
     typedef unsigned int LLVMLinkerMode;
 #endif
 
-/* libc includes */
-#include <stdarg.h> /* for malloc(), free() */
-
 /* Compatibility with Python 2.4: Py_ssize_t is not available. */
 #ifndef PY_SSIZE_T_MAX
 typedef int Py_ssize_t;
 #endif
 
+
+// explicit specializations
+template <> PyObject* pycap_new<int>(int i)
+{
+    return PyLong_FromLong(static_cast<long>(i));
+}
+
+template <> PyObject* pycap_new<unsigned long>(unsigned long i)
+{
+    return PyLong_FromLong(static_cast<unsigned long>(i));
+}
+
+template <> PyObject* pycap_new<LLVMTypeKind>(LLVMTypeKind i)
+{
+    return PyLong_FromLong(static_cast<unsigned long>(i));
+}
+
+template <> PyObject* pycap_new<LLVMLinkage>(LLVMLinkage i)
+{
+    return PyLong_FromLong(static_cast<unsigned long>(i));
+}
+
+template <> PyObject* pycap_new<LLVMVisibility>(LLVMVisibility i)
+{
+    return PyLong_FromLong(static_cast<unsigned long>(i));
+}
+
+template <> PyObject* pycap_new<LLVMByteOrdering>(LLVMByteOrdering i)
+{
+    return PyLong_FromLong(static_cast<unsigned long>(i));
+}
+
+template <> PyObject* pycap_new<unsigned long long>(unsigned long long ull)
+{
+    return PyLong_FromUnsignedLongLong(ull);
+}
+
+template <> PyObject* pycap_new<long long>(long long ll)
+{
+    return PyLong_FromLongLong(ll);
+}
+
 /*===----------------------------------------------------------------------===*/
 /* Modules                                                                    */
 /*===----------------------------------------------------------------------===*/
@@ -66,15 +105,16 @@ typedef int Py_ssize_t;
 static PyObject *
 _wLLVMModuleCreateWithName(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     const char *s;
-    LLVMModuleRef module;
 
     if (!PyArg_ParseTuple(args, "s:LLVMModuleCreateWithName", &s)) {
         return NULL;
     }
 
-    module = LLVMModuleCreateWithName(s);
-    return ctor_LLVMModuleRef(module);
+    const LLVMModuleRef module = LLVMModuleCreateWithName(s);
+    return pycap_new<LLVMModuleRef>(module);
+    LLVMPY_CATCH_ALL
 }
 
 _wrap_obj2str(LLVMGetDataLayout, LLVMModuleRef)
@@ -88,23 +128,21 @@ _wrap_objstr2obj(LLVMGetTypeByName, LLVMModuleRef, LLVMTypeRef)
 _wrap_obj2none(LLVMDumpModule, LLVMModuleRef)
 _wrap_obj2none(LLVMDisposeModule, LLVMModuleRef)
 _wrap_dumper(LLVMDumpModuleToString, LLVMModuleRef)
-_wrap_obj2obj(LLVMModuleGetPointerSize, LLVMModuleRef, int)
+ _wrap_obj2obj( LLVMModuleGetPointerSize, LLVMModuleRef, int)
 _wrap_objstrobj2obj(LLVMModuleGetOrInsertFunction, LLVMModuleRef,
                     LLVMTypeRef, LLVMValueRef)
 
 static PyObject *
 _wLLVMVerifyModule(PyObject *self, PyObject *args)
 {
-    char *outmsg;
-    PyObject *ret;
-    LLVMModuleRef m;
+    LLVMPY_TRY
+    const LLVMModuleRef m = get_object_arg<LLVMModuleRef>(args) ;
+    if (!m) return NULL;
 
-    if (!(m = (LLVMModuleRef)get_object_arg(args)))
-        return NULL;
-
-    outmsg = 0;
+    char *outmsg = 0;
     (void) LLVMVerifyModule(m, LLVMReturnStatusAction, &outmsg);
 
+    PyObject *ret;
     if (outmsg) {
         ret = PyUnicode_FromString(outmsg);
         LLVMDisposeMessage(outmsg);
@@ -113,6 +151,7 @@ _wLLVMVerifyModule(PyObject *self, PyObject *args)
     }
 
     return ret;
+    LLVMPY_CATCH_ALL
 }
 
 ///// unused
@@ -122,13 +161,18 @@ _wLLVMVerifyModule(PyObject *self, PyObject *args)
 static PyObject*
 _wLLVMGetModuleFromAssembly(PyObject *self, PyObject *args)
 {
-    char * str;
+    LLVMPY_TRY
+    const char * str;
+#if (PY_MAJOR_VERSION >= 3)
+    if ( !PyArg_ParseTuple(args, "y", &str) ){
+#else
     if ( !PyArg_ParseTuple(args, "s", &str) ){
+#endif
         return NULL;
     }
 
     char * outmsg = 0;
-    LLVMModuleRef mod = LLVMGetModuleFromAssembly(str, &outmsg);
+    const LLVMModuleRef mod = LLVMGetModuleFromAssembly(str, &outmsg);
 
     if ( !mod ){
         if ( outmsg ){
@@ -140,25 +184,26 @@ _wLLVMGetModuleFromAssembly(PyObject *self, PyObject *args)
         }
     }
 
-    return ctor_LLVMModuleRef(mod);
+    return pycap_new<LLVMModuleRef>(mod);
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMGetModuleFromBitcode(PyObject *self, PyObject *args)
 {
-    PyObject *obj, *ret;
-    Py_ssize_t len;
-    char *start, *outmsg;
-    LLVMModuleRef m;
+    LLVMPY_TRY
+    PyObject *obj ;
 
     if (!PyArg_ParseTuple(args, "S", &obj))
         return NULL;
 
-    start = PyBytes_AsString(obj);
-    len = PyBytes_Size(obj);
+    const char *start = PyBytes_AsString(obj);
+    const Py_ssize_t len = PyBytes_Size(obj);
 
-    outmsg = 0;
-    m = LLVMGetModuleFromBitcode(start, len, &outmsg);
+    char *outmsg = 0;
+    LLVMModuleRef m = LLVMGetModuleFromBitcode(start, len, &outmsg);
+
+    PyObject *ret;
     if (!m) {
         if (outmsg) {
             ret = PyUnicode_FromString(outmsg);
@@ -169,69 +214,72 @@ _wLLVMGetModuleFromBitcode(PyObject *self, PyObject *args)
         }
     }
 
-    return ctor_LLVMModuleRef(m);
+    return pycap_new<LLVMModuleRef>(m);
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMGetBitcodeFromModule(PyObject *self, PyObject *args)
 {
-    PyObject *ret;
-    unsigned len;
-    unsigned char *bytes;
-    LLVMModuleRef m;
+    LLVMPY_TRY
+    const LLVMModuleRef m = get_object_arg<LLVMModuleRef>(args) ;
+    if (!m ) return NULL;
 
-    if (!(m = (LLVMModuleRef)get_object_arg(args)))
-        return NULL;
+    size_t len;
+    unsigned const char *ubytes = LLVMGetBitcodeFromModule(m, &len) ;
+    if ( !ubytes ) Py_RETURN_NONE;
 
-    if (!(bytes = LLVMGetBitcodeFromModule(m, &len)))
-        Py_RETURN_NONE;
-
-    ret = PyBytes_FromStringAndSize((char *)bytes, (Py_ssize_t)len);
-    LLVMDisposeMessage((char *)bytes);
+    const char *chars = reinterpret_cast<const char*>(ubytes) ;
+    PyObject *ret = PyBytes_FromStringAndSize(chars, len);
+    LLVMDisposeMessage(const_cast<char*>(chars));
     return ret;
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMGetNativeCodeFromModule(PyObject * self, PyObject * args)
 {
-    PyObject * ret;
-    unsigned len;
-    unsigned char * bytes;
-    LLVMModuleRef m;
-
+    LLVMPY_TRY
     PyObject * arg_m;
     int arg_use_asm;
 
     if (!PyArg_ParseTuple(args, "Oi", &arg_m, &arg_use_asm))
         return NULL;
 
-    m = (LLVMModuleRef) PyCapsule_GetPointer(arg_m, NULL);
+    const LLVMModuleRef m = pycap_get<LLVMModuleRef>( arg_m ) ;
 
     std::string error;
-    bytes = LLVMGetNativeCodeFromModule(m, arg_use_asm, &len, error);
+    size_t len;
+    unsigned const char * ubytes
+        = LLVMGetNativeCodeFromModule(m, arg_use_asm, &len, error);
     if ( !error.empty() ){
         PyErr_SetString(PyExc_RuntimeError, error.c_str());
     }
 
-    ret = PyBytes_FromStringAndSize((char *)bytes, (Py_ssize_t)len);
-    delete [] bytes;
+    const char *chars = reinterpret_cast<const char*>(ubytes) ;
+    PyObject * ret = PyBytes_FromStringAndSize(chars, len);
+
+    LLVMDisposeMessage(const_cast<char*>(chars));
+
     return ret;
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMLinkModules(PyObject *self, PyObject *args)
 {
-    PyObject *dest_obj, *src_obj, *ret;
-    LLVMModuleRef dest, src;
-    char *errmsg;
-    unsigned int mode = 0;
+    LLVMPY_TRY
+    size_t mode = 0;
 
+    PyObject *dest_obj, *src_obj ;
     if (!PyArg_ParseTuple(args, "OO|I", &dest_obj, &src_obj, &mode))
         return NULL;
 
-    dest = (LLVMModuleRef) PyCapsule_GetPointer(dest_obj, NULL);
-    src = (LLVMModuleRef) PyCapsule_GetPointer(src_obj, NULL);
+    const LLVMModuleRef dest = pycap_get<LLVMModuleRef>(dest_obj ) ;
+    const LLVMModuleRef src = pycap_get<LLVMModuleRef>(src_obj) ;
 
+    PyObject *ret;
+    char *errmsg;
     if (!LLVMLinkModules(dest, src, (LLVMLinkerMode)mode, &errmsg)) {
         if (errmsg) {
             ret = PyUnicode_FromString(errmsg);
@@ -244,8 +292,13 @@ _wLLVMLinkModules(PyObject *self, PyObject *args)
 
     /* note: success => None, failure => string with error message */
     Py_RETURN_NONE;
+    LLVMPY_CATCH_ALL
 }
 
+_wrap_objstr2obj(LLVMModuleGetOrInsertNamedMetaData, LLVMModuleRef, LLVMNamedMDRef)
+_wrap_objstr2obj(LLVMModuleGetNamedMetaData, LLVMModuleRef, LLVMNamedMDRef)
+_wrap_obj2obj(LLVMCloneModule, LLVMModuleRef, LLVMModuleRef)
+
 /*===----------------------------------------------------------------------===*/
 /* Types                                                                      */
 /*===----------------------------------------------------------------------===*/
@@ -291,29 +344,25 @@ typedef unsigned (*arrcnt_fn_t)(LLVMTypeRef ty);
 static PyObject *
 obj2arr(PyObject *self, PyObject *args, arrcnt_fn_t cntfunc, obj2arr_fn_t arrfunc)
 {
-    LLVMTypeRef type, *param_types;
-    unsigned param_count;
-    PyObject *list;
+    size_t param_count;
 
     /* get the function object ptr */
-    if (!(type = (LLVMTypeRef)get_object_arg(args)))
-        return NULL;
+    const LLVMTypeRef type = get_object_arg<LLVMTypeRef>(args) ;
 
     /* get param count */
     param_count = cntfunc(type);
 
     /* alloc enough space for all of them */
-    if (!(param_types = (LLVMTypeRef *)malloc(sizeof(LLVMTypeRef) * param_count)))
-        return PyErr_NoMemory();
+    LLVMTypeRef* param_types = new LLVMTypeRef[param_count] ;
 
     /* call LLVM func */
     arrfunc(type, param_types);
 
     /* create a list from the array */
-    list = make_list_from_LLVMTypeRef_array(param_types, param_count);
+    PyObject *list = make_list<LLVMTypeRef>(param_types, param_count);
 
     /* free temp storage */
-    free(param_types);
+    delete [] param_types ;
 
     return list;
 }
@@ -321,7 +370,9 @@ obj2arr(PyObject *self, PyObject *args, arrcnt_fn_t cntfunc, obj2arr_fn_t arrfun
 static PyObject *
 _wLLVMGetFunctionTypeParams(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     return obj2arr(self, args, LLVMCountParamTypes, LLVMGetParamTypes);
+    LLVMPY_CATCH_ALL
 }
 
 /*===-- Struct types -----------------------------------------------------===*/
@@ -337,7 +388,10 @@ _wrap_objstr2none(LLVMSetStructName, LLVMTypeRef)
 static PyObject *
 _wLLVMGetStructElementTypes(PyObject *self, PyObject *args)
 {
-    return obj2arr(self, args, LLVMCountStructElementTypes, LLVMGetStructElementTypes);
+    LLVMPY_TRY
+    return obj2arr(self, args, LLVMCountStructElementTypes,
+            LLVMGetStructElementTypes);
+    LLVMPY_CATCH_ALL
 }
 
 
@@ -392,19 +446,18 @@ _wrap_obj2obj(LLVMValueGetNumUses, LLVMValueRef, int)
 static PyObject *
 _wLLVMValueGetUses(PyObject *self, PyObject *args)
 {
-    LLVMValueRef value;
-
-    if (!(value = (LLVMValueRef)get_object_arg(args)))
-        return NULL;
+    LLVMPY_TRY
+    const LLVMValueRef value = get_object_arg<LLVMValueRef>(args) ;
 
     LLVMValueRef *uses = 0;
-    unsigned n = LLVMValueGetUses(value, &uses);
+    size_t n = LLVMValueGetUses(value, &uses);
 
-    PyObject *list = make_list_from_LLVMValueRef_array(uses, n);
+    PyObject *list = make_list<LLVMValueRef>(uses, n);
     if (n > 0)
         LLVMDisposeValueRefArray(uses);
 
     return list;
+    LLVMPY_CATCH_ALL
 }
 
 /*===-- Users ------------------------------------------------------------===*/
@@ -428,55 +481,59 @@ _wrap_obj2obj(LLVMIsUndef, LLVMValueRef, int)
 static PyObject *
 _wLLVMConstInt(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     PyObject *obj;
     unsigned long long n;
     int sign_extend;
-    LLVMTypeRef ty;
-    LLVMValueRef val;
 
     if (!PyArg_ParseTuple(args, "OKi", &obj, &n, &sign_extend))
         return NULL;
 
-    ty = (LLVMTypeRef) PyCapsule_GetPointer(obj, NULL);
+    const LLVMTypeRef ty = pycap_get<LLVMTypeRef>( obj ) ;
 
-    val = LLVMConstInt(ty, n, sign_extend);
-    return ctor_LLVMValueRef(val);
+    const LLVMValueRef val = LLVMConstInt(ty, n, sign_extend);
+
+    return pycap_new<LLVMValueRef>(val);
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMConstReal(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     PyObject *obj;
     double d;
-    LLVMTypeRef ty;
-    LLVMValueRef val;
 
     if (!PyArg_ParseTuple(args, "Od", &obj, &d))
         return NULL;
 
-    ty = (LLVMTypeRef) PyCapsule_GetPointer(obj, NULL);
+    const LLVMTypeRef ty = pycap_get<LLVMTypeRef>( obj ) ;
 
-    val = LLVMConstReal(ty, d);
-    return ctor_LLVMValueRef(val);
+    const LLVMValueRef val = LLVMConstReal(ty, d);
+    return pycap_new<LLVMValueRef>(val);
+    LLVMPY_CATCH_ALL
 }
 
 _wrap_objstr2obj(LLVMConstRealOfString, LLVMTypeRef, LLVMValueRef)
+_wrap_obj2obj(LLVMConstIntGetZExtValue, LLVMValueRef, llvmwrap_ull)
+_wrap_obj2obj(LLVMConstIntGetSExtValue, LLVMValueRef, llvmwrap_ll)
 
 /* Operations on composite constants */
 
 static PyObject *
 _wLLVMConstString(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     const char *s;
     int dont_null_terminate;
-    LLVMValueRef val;
 
     if (!PyArg_ParseTuple(args, "si:LLVMConstString", &s, &dont_null_terminate)) {
         return NULL;
     }
 
-    val = LLVMConstString(s, strlen(s), dont_null_terminate);
-    return ctor_LLVMValueRef(val);
+    LLVMValueRef val = LLVMConstString(s, strlen(s), dont_null_terminate);
+    return pycap_new<LLVMValueRef>(val);
+    LLVMPY_CATCH_ALL
 }
 
 _wrap_objlist2obj(LLVMConstArray, LLVMTypeRef, LLVMValueRef, LLVMValueRef)
@@ -485,6 +542,8 @@ _wrap_list2obj(LLVMConstVector, LLVMValueRef, LLVMValueRef)
 
 /* Constant expressions */
 
+_wrap_obj2obj(LLVMGetConstExprOpcode, LLVMValueRef, int)
+_wrap_obj2str(LLVMGetConstExprOpcodeName, LLVMValueRef)
 _wrap_obj2obj(LLVMSizeOf, LLVMTypeRef, LLVMValueRef)
 _wrap_obj2obj(LLVMConstNeg, LLVMValueRef, LLVMValueRef)
 _wrap_obj2obj(LLVMConstNot, LLVMValueRef, LLVMValueRef)
@@ -548,8 +607,9 @@ _wrap_obj2obj(LLVMGetAlignment, LLVMValueRef, int)
 _wrap_objint2none(LLVMSetAlignment, LLVMValueRef)
 
 /*===-- Global Variables -------------------------------------------------===*/
-
+    
 _wrap_objobjstr2obj(LLVMAddGlobal, LLVMModuleRef, LLVMTypeRef, LLVMValueRef)
+_wrap_objobjstrint2obj(LLVMAddGlobalInAddressSpace, LLVMModuleRef, LLVMTypeRef, LLVMValueRef)
 _wrap_objstr2obj(LLVMGetNamedGlobal, LLVMModuleRef, LLVMValueRef)
 _wrap_obj2obj(LLVMGetFirstGlobal, LLVMModuleRef, LLVMValueRef)
 _wrap_obj2obj(LLVMGetNextGlobal, LLVMValueRef, LLVMValueRef)
@@ -557,10 +617,11 @@ _wrap_obj2none(LLVMDeleteGlobal, LLVMValueRef)
 _wrap_obj2obj(LLVMHasInitializer, LLVMValueRef, int)
 _wrap_obj2obj(LLVMGetInitializer, LLVMValueRef, LLVMValueRef)
 _wrap_objobj2none(LLVMSetInitializer, LLVMValueRef, LLVMValueRef)
-_wrap_objint2none(LLVMSetThreadLocal, LLVMValueRef)
 _wrap_objint2none(LLVMSetGlobalConstant, LLVMValueRef)
-_wrap_obj2obj(LLVMIsThreadLocal, LLVMValueRef, int)
 _wrap_obj2obj(LLVMIsGlobalConstant, LLVMValueRef, int)
+_wrap_objint2none(LLVMSetThreadLocal, LLVMValueRef)
+_wrap_obj2obj(LLVMIsThreadLocal, LLVMValueRef, int)
+
 
 /*===-- Functions --------------------------------------------------------===*/
 
@@ -581,16 +642,42 @@ _wrap_obj2none(LLVMViewFunctionCFGOnly, LLVMValueRef)
 _wrap_objenum2none(LLVMAddFunctionAttr, LLVMValueRef, LLVMAttribute)
 _wrap_objenum2none(LLVMRemoveFunctionAttr, LLVMValueRef, LLVMAttribute)
 
-
 static PyObject *
 _wLLVMVerifyFunction(PyObject *self, PyObject *args)
 {
-    LLVMValueRef fn;
+    LLVMPY_TRY
+    const LLVMValueRef fn = get_object_arg<LLVMValueRef>(args);
+    if (!fn) return NULL;
 
-    if (!(fn = (LLVMValueRef)get_object_arg(args)))
+    return pycap_new<int>(LLVMVerifyFunction(fn, LLVMReturnStatusAction));
+    LLVMPY_CATCH_ALL
+}
+
+static PyObject *
+_wLLVMGetFunctionFromInlineAsm(PyObject *self, PyObject *args)
+{
+    LLVMPY_TRY
+    PyObject *pycapFuncType;
+    const char *inlineAsm = NULL;
+    const char *constrains = NULL;
+    int hasSideEffect;
+    int isAlignStack;
+    int asmDialect;
+
+    if (!PyArg_ParseTuple(args, "Ossiii", &pycapFuncType, &inlineAsm,
+                          &constrains, &hasSideEffect,
+                          &isAlignStack, &asmDialect))
         return NULL;
 
-    return ctor_int(LLVMVerifyFunction(fn, LLVMReturnStatusAction));
+
+    LLVMTypeRef funcType = pycap_get<LLVMTypeRef>(pycapFuncType);
+    LLVMValueRef inlineAsmRef;
+    inlineAsmRef = LLVMGetFunctionFromInlineAsm(funcType, inlineAsm, constrains,
+                                                hasSideEffect, isAlignStack,
+                                                asmDialect);
+
+    return pycap_new<LLVMValueRef>(inlineAsmRef);
+    LLVMPY_CATCH_ALL
 }
 
 
@@ -616,6 +703,45 @@ _wrap_objstr2obj(LLVMAppendBasicBlock, LLVMValueRef, LLVMBasicBlockRef)
 _wrap_objstr2obj(LLVMInsertBasicBlock, LLVMBasicBlockRef, LLVMBasicBlockRef)
 _wrap_obj2none(LLVMDeleteBasicBlock, LLVMBasicBlockRef)
 
+
+/*===-- MetaData -----------------------------------------------------===*/
+
+_wrap_objlist2obj(LLVMMetaDataGet, LLVMModuleRef, LLVMValueRef, LLVMValueRef)
+_wrap_objstrobj2none(LLVMAddNamedMetadataOperand, LLVMModuleRef, LLVMValueRef)
+_wrap_objint2obj(LLVMMetaDataGetOperand, LLVMValueRef, LLVMValueRef)
+_wrap_obj2obj(LLVMMetaDataGetNumOperands, LLVMValueRef, int)
+_wrap_objstr2obj(LLVMMetaDataStringGet, LLVMModuleRef, LLVMValueRef)
+
+static PyObject *
+_wLLVMGetNamedMetadataOperands(PyObject *self, PyObject *args)
+{
+    LLVMPY_TRY
+    PyObject *obj_module;
+    const char *name;
+
+    if (!PyArg_ParseTuple(args, "Os", &obj_module, &name))
+        return NULL;
+
+    LLVMModuleRef module = pycap_get<LLVMModuleRef>(obj_module);
+    unsigned num_operands = LLVMGetNamedMetadataNumOperands(module, name);
+    LLVMValueRef *operands = new LLVMValueRef[num_operands];
+    LLVMGetNamedMetadataOperands(module, name, operands);
+
+    PyObject *list = make_list<LLVMValueRef>(operands, num_operands);
+    delete [] operands;
+
+    return list;
+    LLVMPY_CATCH_ALL
+}
+
+
+/*===-- NamedMetaData -----------------------------------------------------===*/
+
+_wrap_obj2str( LLVMNamedMetaDataGetName, LLVMNamedMDRef )
+_wrap_objobj2none( LLVMNamedMetaDataAddOperand, LLVMNamedMDRef, LLVMValueRef )
+_wrap_obj2none( LLVMEraseNamedMetaData, LLVMNamedMDRef )
+_wrap_dumper(LLVMDumpNamedMDToString, LLVMNamedMDRef)
+
 /*===-- Instructions -----------------------------------------------------===*/
 
 _wrap_obj2obj(LLVMGetInstructionParent, LLVMValueRef, LLVMBasicBlockRef)
@@ -634,6 +760,8 @@ _wrap_objint2none(LLVMSetVolatile,    LLVMValueRef)
 _wrap_obj2obj(LLVMInstGetOpcode,      LLVMValueRef, int)
 _wrap_obj2str(LLVMInstGetOpcodeName,  LLVMValueRef)
 
+_wrap_objstrobj2none(LLVMInstSetMetaData, LLVMValueRef, LLVMValueRef)
+
 /*===-- Call Sites (Call or Invoke) --------------------------------------===*/
 
 _wrap_objint2none(LLVMSetInstructionCallConv, LLVMValueRef)
@@ -643,7 +771,8 @@ _wrap_objintenum2none(LLVMRemoveInstrAttribute, LLVMValueRef, LLVMAttribute)
 _wrap_objintint2none(LLVMSetInstrParamAlignment, LLVMValueRef)
 _wrap_obj2obj(LLVMIsTailCall, LLVMValueRef, int)
 _wrap_objint2none(LLVMSetTailCall, LLVMValueRef)
-
+_wrap_obj2obj(LLVMInstGetCalledFunction, LLVMValueRef, LLVMValueRef)
+_wrap_objobj2none(LLVMInstSetCalledFunction, LLVMValueRef, LLVMValueRef)
 
 /*===-- PHI Nodes --------------------------------------------------------===*/
 
@@ -681,37 +810,32 @@ _wrap_objobjobjint2obj(LLVMBuildSwitch, LLVMBuilderRef, LLVMValueRef, LLVMBasicB
 static PyObject *
 _wLLVMBuildInvoke(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     PyObject *obj1, *obj2, *obj3, *obj4, *obj5;
     const char *name;
-    LLVMBuilderRef builder;
-    LLVMValueRef func;
-    LLVMValueRef *fnargs;
-    unsigned fnarg_count;
-    LLVMBasicBlockRef then_blk, catch_blk;
-    LLVMValueRef inst;
 
     if (!PyArg_ParseTuple(args, "OOOOOs:LLVMBuildInvoke", &obj1, &obj2, &obj3, &obj4, &obj5, &name)) {
         return NULL;
     }
 
-    builder = (LLVMBuilderRef) PyCapsule_GetPointer(obj1, NULL);
-    func = (LLVMValueRef) PyCapsule_GetPointer(obj2, NULL);
+    const LLVMBuilderRef builder = pycap_get<LLVMBuilderRef>( obj1);
+    const LLVMValueRef func = pycap_get<LLVMValueRef>( obj2 ) ;
 
+    size_t fnarg_count = PyList_Size(obj3);
+    LLVMValueRef *fnargs
+        = make_array_from_list<LLVMValueRef *>(obj3, fnarg_count);
 
-    fnarg_count = (unsigned) PyList_Size(obj3);
-    fnargs = (LLVMValueRef *)make_array_from_list(obj3, fnarg_count);
-    if (!fnargs)
-        return PyErr_NoMemory();
+    const LLVMBasicBlockRef then_blk = pycap_get<LLVMBasicBlockRef> ( obj4 ) ;
+    const LLVMBasicBlockRef catch_blk = pycap_get<LLVMBasicBlockRef> ( obj5 ) ;
 
-    then_blk = (LLVMBasicBlockRef) PyCapsule_GetPointer(obj4, NULL);
-    catch_blk = (LLVMBasicBlockRef) PyCapsule_GetPointer(obj5, NULL);
+    const LLVMValueRef inst
+        = LLVMBuildInvoke(builder, func, fnargs, fnarg_count, then_blk,
+                catch_blk, name);
 
+    delete [] fnargs;
 
-    inst = LLVMBuildInvoke(builder, func, fnargs, fnarg_count, then_blk, catch_blk, name);
-
-    free(fnargs);
-
-    return ctor_LLVMValueRef(inst);
+    return pycap_new<LLVMValueRef>(inst);
+    LLVMPY_CATCH_ALL
 }
 
 _wrap_obj2obj(LLVMBuildUnreachable, LLVMBuilderRef, LLVMValueRef)
@@ -805,40 +929,43 @@ _wrap_objobjobjobjstr2obj(LLVMBuildShuffleVector, LLVMBuilderRef, LLVMValueRef,
 static PyObject *
 _wLLVMCreateMemoryBufferWithContentsOfFile(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     const char *path;
     LLVMMemoryBufferRef ref;
-    char *outmsg;
     PyObject *ret;
 
     if (!PyArg_ParseTuple(args, "s:LLVMCreateMemoryBufferWithContentsOfFile", &path)) {
         return NULL;
     }
 
+    char *outmsg;
     if (!LLVMCreateMemoryBufferWithContentsOfFile(path, &ref, &outmsg)) {
-        ret = ctor_LLVMMemoryBufferRef(ref);
+        ret = pycap_new<LLVMMemoryBufferRef>(ref);
     } else {
         ret = PyUnicode_FromString(outmsg);
         LLVMDisposeMessage(outmsg);
     }
 
     return ret;
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMCreateMemoryBufferWithSTDIN(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
+    PyObject *ret;
     LLVMMemoryBufferRef ref;
     char *outmsg;
-    PyObject *ret;
-
     if (!LLVMCreateMemoryBufferWithSTDIN(&ref, &outmsg)) {
-        ret = ctor_LLVMMemoryBufferRef(ref);
+        ret = pycap_new<LLVMMemoryBufferRef>(ref);
     } else {
         ret = PyUnicode_FromString(outmsg);
         LLVMDisposeMessage(outmsg);
     }
 
     return ret;
+    LLVMPY_CATCH_ALL
 }
 
 _wrap_obj2none(LLVMDisposeMemoryBuffer, LLVMMemoryBufferRef)
@@ -1029,6 +1156,7 @@ _wrap_obj2none(LLVMDisposeTargetMachine, LLVMTargetMachineRef)
 static PyObject *
 _wLLVMTargetMachineLookup(PyObject * self, PyObject * args)
 {
+    LLVMPY_TRY
     const char *arch;
     const char *cpu;
     const char *features;
@@ -1044,37 +1172,38 @@ _wLLVMTargetMachineLookup(PyObject * self, PyObject * args)
         PyErr_SetString(PyExc_RuntimeError, error.c_str());
         return NULL;
     }
-    return ctor_LLVMTargetMachineRef(tm);
+    return pycap_new<LLVMTargetMachineRef>(tm);
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMTargetMachineEmitFile(PyObject * self, PyObject * args)
 {
-    PyObject * ret;
-    unsigned len;
-    unsigned char * bytes;
-    LLVMTargetMachineRef tm;
-    LLVMModuleRef m;
-
+    LLVMPY_TRY
     PyObject *arg_tm, *arg_m;
     int arg_use_asm;
 
     if (!PyArg_ParseTuple(args, "OOi", &arg_tm, &arg_m, &arg_use_asm))
         return NULL;
 
-    tm = (LLVMTargetMachineRef) PyCapsule_GetPointer(arg_tm, NULL);
-    m = (LLVMModuleRef) PyCapsule_GetPointer(arg_m, NULL);
+    const LLVMTargetMachineRef tm
+        = pycap_get<LLVMTargetMachineRef>( arg_tm ) ;
+    const LLVMModuleRef m = pycap_get<LLVMModuleRef>( arg_m ) ;
 
     std::string error;
-    bytes = LLVMTargetMachineEmitFile(tm, m, arg_use_asm, &len, error);
+    size_t len;
+    unsigned const char * ubytes
+        = LLVMTargetMachineEmitFile(tm, m, arg_use_asm, &len, error);
     if ( !error.empty() ){
         PyErr_SetString(PyExc_RuntimeError, error.c_str());
         return NULL;
     }
 
-    ret = PyBytes_FromStringAndSize((char *)bytes, (Py_ssize_t)len);
-    delete [] bytes;
+    const char *chars = reinterpret_cast<const char*>(ubytes) ;
+    PyObject * ret = PyBytes_FromStringAndSize(chars, len);
+    delete [] ubytes;
     return ret;
+    LLVMPY_CATCH_ALL
 }
 
 _wrap_obj2obj(LLVMTargetMachineGetTargetData, LLVMTargetMachineRef,
@@ -1097,17 +1226,14 @@ _wrap_obj2none(LLVMDisposeTargetData, LLVMTargetDataRef)
 static PyObject *
 _wLLVMTargetDataAsString(PyObject *self, PyObject *args)
 {
-    LLVMTargetDataRef td;
-    char *tdrep = 0;
-    PyObject *ret;
+    LLVMPY_TRY
+    const LLVMTargetDataRef td = get_object_arg<LLVMTargetDataRef>(args);
 
-    if (!(td = (LLVMTargetDataRef)get_object_arg(args)))
-        return NULL;
-
-    tdrep = LLVMCopyStringRepOfTargetData(td);
-    ret = PyUnicode_FromString(tdrep);
+    char *tdrep = LLVMCopyStringRepOfTargetData(td);
+    PyObject *ret = PyUnicode_FromString(tdrep);
     LLVMDisposeMessage(tdrep);
     return ret;
+    LLVMPY_CATCH_ALL
 }
 
 _wrap_objobj2none(LLVMAddTargetData, LLVMTargetDataRef, LLVMPassManagerRef)
@@ -1148,22 +1274,22 @@ _wrap_objstr2none(LLVMEngineBuilderSetMAttrs, LLVMEngineBuilderRef)
 static PyObject *
 _wLLVMEngineBuilderCreate(PyObject *self, PyObject *args)
 {
-    LLVMEngineBuilderRef obj;
-    if (!(obj = (LLVMEngineBuilderRef)get_object_arg(args)))
-        return NULL;
+    LLVMPY_TRY
+    const LLVMEngineBuilderRef obj
+        = get_object_arg<LLVMEngineBuilderRef>(args) ;
 
     std::string outmsg;
-
-    LLVMExecutionEngineRef ee = LLVMEngineBuilderCreate(obj, outmsg);
+    const LLVMExecutionEngineRef ee = LLVMEngineBuilderCreate(obj, outmsg);
 
     PyObject * ret;
     if( !ee ){ // check if error message is set.
         ret = PyUnicode_FromString(outmsg.c_str());
     }else{
-        ret = ctor_LLVMExecutionEngineRef(ee);
+        ret = pycap_new<LLVMExecutionEngineRef>(ee);
     }
 
     return ret;
+    LLVMPY_CATCH_ALL
 }
 
 /*===----------------------------------------------------------------------===*/
@@ -1173,49 +1299,50 @@ _wLLVMEngineBuilderCreate(PyObject *self, PyObject *args)
 static PyObject *
 _wLLVMCreateExecutionEngine(PyObject *self, PyObject *args)
 {
-    LLVMModuleRef mod;
+    LLVMPY_TRY
     PyObject *obj;
     int force_interpreter;
-    LLVMExecutionEngineRef ee;
     char *outmsg = 0;
-    PyObject *ret;
     int error;
 
     if (!PyArg_ParseTuple(args, "Oi", &obj, &force_interpreter))
         return NULL;
 
-    mod = (LLVMModuleRef) PyCapsule_GetPointer(obj, NULL);
+    const LLVMModuleRef mod = pycap_get<LLVMModuleRef>( obj ) ;
 
+    LLVMExecutionEngineRef ee;
     if (force_interpreter)
         error = LLVMCreateInterpreterForModule(&ee, mod, &outmsg);
     else
         error = LLVMCreateJITCompilerForModule(&ee, mod, 1 /*fast*/, &outmsg);
 
+    PyObject *ret;
     if (error) {
         ret = PyUnicode_FromString(outmsg);
         LLVMDisposeMessage(outmsg);
     } else {
-        ret = ctor_LLVMExecutionEngineRef(ee);
+        ret = pycap_new<LLVMExecutionEngineRef>(ee);
     }
 
     return ret;
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMGetPointerToFunction(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     PyObject *obj_ee;
     PyObject *obj_fn;
-    LLVMExecutionEngineRef ee;
-    LLVMValueRef fn;
 
     if (!PyArg_ParseTuple(args, "OO", &obj_ee, &obj_fn))
         return NULL;
 
-    ee = (LLVMExecutionEngineRef) PyCapsule_GetPointer(obj_ee, NULL);
-    fn = (LLVMValueRef) PyCapsule_GetPointer(obj_fn, NULL);
+    const LLVMExecutionEngineRef ee = pycap_get<LLVMExecutionEngineRef>(obj_ee ) ;
+    const LLVMValueRef fn = pycap_get<LLVMValueRef>( obj_fn ) ;
 
     return PyLong_FromVoidPtr(LLVMGetPointerToFunction(ee,fn));
+    LLVMPY_CATCH_ALL
 }
 
 /* the args should have been ptr, num */
@@ -1228,22 +1355,21 @@ LLVMGenericValueRef LLVMRunFunction2(LLVMExecutionEngineRef EE,
 static PyObject *
 _wLLVMRemoveModule2(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     PyObject *obj_ee;
     PyObject *obj_mod;
-    LLVMExecutionEngineRef ee;
-    LLVMModuleRef mod, mod_new = 0;
-    char *outmsg = 0;
-    PyObject *ret;
-
     if (!PyArg_ParseTuple(args, "OO", &obj_ee, &obj_mod))
         return NULL;
 
-    ee = (LLVMExecutionEngineRef) PyCapsule_GetPointer(obj_ee, NULL);
-    mod = (LLVMModuleRef) PyCapsule_GetPointer(obj_mod, NULL);
+    const LLVMExecutionEngineRef ee = pycap_get<LLVMExecutionEngineRef>( obj_ee ) ;
+    const LLVMModuleRef mod = pycap_get<LLVMModuleRef>(obj_mod) ;
 
+    char *outmsg = 0;
+    LLVMModuleRef mod_new = 0;
     LLVMRemoveModule(ee, mod, &mod_new, &outmsg);
+    PyObject *ret;
     if (mod_new) {
-        ret = ctor_LLVMModuleRef(mod_new);
+        ret = pycap_new<LLVMModuleRef>(mod_new);
     } else {
         if (outmsg) {
             ret = PyUnicode_FromString(outmsg);
@@ -1254,6 +1380,7 @@ _wLLVMRemoveModule2(PyObject *self, PyObject *args)
     }
 
     return ret;
+    LLVMPY_CATCH_ALL
 }
 
 _wrap_obj2none(LLVMDisposeExecutionEngine, LLVMExecutionEngineRef)
@@ -1276,8 +1403,8 @@ _wrap_objobj2none(LLVMAddModule, LLVMExecutionEngineRef,
 static PyObject *
 _wLLVMCreateGenericValueOfInt(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     PyObject *obj1;
-    LLVMTypeRef ty;
     unsigned long long n;
     int is_signed;
     LLVMGenericValueRef gv;
@@ -1285,100 +1412,101 @@ _wLLVMCreateGenericValueOfInt(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "OLi", &obj1, &n, &is_signed))
         return NULL;
 
-    ty = (LLVMTypeRef) PyCapsule_GetPointer(obj1, NULL);
+    const LLVMTypeRef ty = pycap_get<LLVMTypeRef>( obj1 ) ;
 
     gv = LLVMCreateGenericValueOfInt(ty, n, is_signed);
-    return ctor_LLVMGenericValueRef(gv);
+    return pycap_new<LLVMGenericValueRef>(gv);
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMCreateGenericValueOfFloat(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     PyObject *obj1;
-    LLVMTypeRef ty;
     double d;
     LLVMGenericValueRef gv;
 
     if (!PyArg_ParseTuple(args, "Od", &obj1, &d))
         return NULL;
 
-    ty = (LLVMTypeRef) PyCapsule_GetPointer(obj1, NULL);
+    const LLVMTypeRef ty = pycap_get<LLVMTypeRef>( obj1 ) ;
 
     gv = LLVMCreateGenericValueOfFloat(ty, d);
-    return ctor_LLVMGenericValueRef(gv);
+    return pycap_new<LLVMGenericValueRef>(gv);
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMCreateGenericValueOfPointer(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
 //    PyObject *obj1;
 //    LLVMTypeRef ty; //unused?
     unsigned long long n_;
     size_t n;
-    LLVMGenericValueRef gv;
 
     if (!PyArg_ParseTuple(args, "L", &n_))
         return NULL;
 
     n=n_;
 
-//    ty = (LLVMTypeRef) PyCapsule_GetPointer(obj1, NULL);
-
-    gv = LLVMCreateGenericValueOfPointer((void*)n);
-    return ctor_LLVMGenericValueRef(gv);
+    const LLVMGenericValueRef gv = LLVMCreateGenericValueOfPointer((void*)n);
+    return pycap_new<LLVMGenericValueRef>(gv);
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMGenericValueToInt(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     PyObject *obj1;
     int is_signed;
-    LLVMGenericValueRef gv;
     unsigned long long val;
 
     if (!PyArg_ParseTuple(args, "Oi", &obj1, &is_signed))
         return NULL;
 
-    gv = (LLVMGenericValueRef) PyCapsule_GetPointer(obj1, NULL);
+    LLVMGenericValueRef gv = pycap_get<LLVMGenericValueRef>(obj1);
 
     val = LLVMGenericValueToInt(gv, is_signed);
     return is_signed ?
         PyLong_FromLongLong((long long)val) :
         PyLong_FromUnsignedLongLong(val);
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMGenericValueToFloat(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     PyObject *obj1, *obj2;
-    LLVMTypeRef ty;
-    LLVMGenericValueRef gv;
-    double val;
 
     if (!PyArg_ParseTuple(args, "OO", &obj1, &obj2))
         return NULL;
 
-    ty = (LLVMTypeRef) PyCapsule_GetPointer(obj1, NULL);
-    gv = (LLVMGenericValueRef) PyCapsule_GetPointer(obj2, NULL);
+    const LLVMTypeRef ty = pycap_get<LLVMTypeRef>( obj1 ) ;
+    const LLVMGenericValueRef gv = pycap_get<LLVMGenericValueRef>( obj2 ) ;
 
-    val = LLVMGenericValueToFloat(ty, gv);
+    double val = LLVMGenericValueToFloat(ty, gv);
     return PyFloat_FromDouble(val);
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMGenericValueToPointer(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     PyObject *obj1;
-    LLVMGenericValueRef gv;
-    void * val;
 
     if (!PyArg_ParseTuple(args, "O", &obj1))
         return NULL;
 
-    gv = (LLVMGenericValueRef) PyCapsule_GetPointer(obj1, NULL);
+    const LLVMGenericValueRef gv = pycap_get<LLVMGenericValueRef>( obj1 ) ;
 
-    val = LLVMGenericValueToPointer(gv);
+    void * val = LLVMGenericValueToPointer(gv);
     return PyLong_FromVoidPtr(val);
+    LLVMPY_CATCH_ALL
 }
 
 _wrap_obj2none(LLVMDisposeGenericValue, LLVMGenericValueRef)
@@ -1394,15 +1522,15 @@ _wrap_objintlist2obj(LLVMGetIntrinsic, LLVMModuleRef, LLVMTypeRef,
 static PyObject *
 _wLLVMLoadLibraryPermanently(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     const char *filename;
-    char *outmsg;
     PyObject *ret;
 
     if (!PyArg_ParseTuple(args, "s:LLVMLoadLibraryPermanently", &filename)) {
         return NULL;
     }
 
-    outmsg = 0;
+    char *outmsg = 0;
     if (!LLVMLoadLibraryPermanently(filename, &outmsg)) {
         if (outmsg) {
             ret = PyUnicode_FromString(outmsg);
@@ -1413,11 +1541,13 @@ _wLLVMLoadLibraryPermanently(PyObject *self, PyObject *args)
 
     /* note: success => None, failure => string with error message */
     Py_RETURN_NONE;
+    LLVMPY_CATCH_ALL
 }
 
 static PyObject *
 _wLLVMParseEnvOpts(PyObject *self, PyObject *args)
 {
+    LLVMPY_TRY
     const char * progname;
     const char * envname;
     if(!PyArg_ParseTuple(args, "ss", &progname, &envname)) {
@@ -1427,21 +1557,20 @@ _wLLVMParseEnvOpts(PyObject *self, PyObject *args)
     llvm::cl::ParseEnvironmentOptions(progname, envname);
 
     Py_RETURN_NONE;
+    LLVMPY_CATCH_ALL
 }
 
-//_wrap_obj2obj(LLVMInlineFunction, LLVMValueRef, int)
+_wrap_obj2obj(LLVMInlineFunction, LLVMValueRef, int)
 
 /* Expose the void* inside a PyCObject as a PyLong. This allows us to
  * use it as a unique ID. */
 static PyObject *
 _wPyCObjectVoidPtrToPyLong(PyObject *self, PyObject *args)
 {
-    void *p;
-
-    if (!(p = get_object_arg(args)))
-        return NULL;
-
+    LLVMPY_TRY
+    void *p = get_object_arg(args);
     return PyLong_FromVoidPtr(p);
+    LLVMPY_CATCH_ALL
 }
 
 
@@ -1475,6 +1604,9 @@ static PyMethodDef core_methods[] = {
     _method( LLVMModuleGetPointerSize )
     _method( LLVMModuleGetOrInsertFunction )
     _method( LLVMLinkModules )
+    _method( LLVMModuleGetOrInsertNamedMetaData )
+    _method( LLVMModuleGetNamedMetaData )
+    _method( LLVMCloneModule )
 
     /* Types */
 
@@ -1571,6 +1703,8 @@ static PyMethodDef core_methods[] = {
     _method( LLVMConstInt )
     _method( LLVMConstReal )
     _method( LLVMConstRealOfString )
+    _method( LLVMConstIntGetZExtValue )
+    _method( LLVMConstIntGetSExtValue )
 
     /* Operations on composite constants */
     _method( LLVMConstString )
@@ -1579,6 +1713,8 @@ static PyMethodDef core_methods[] = {
     _method( LLVMConstVector )
 
     /* Constant expressions */
+    _method( LLVMGetConstExprOpcode )
+    _method( LLVMGetConstExprOpcodeName )
     _method( LLVMSizeOf )
     _method( LLVMConstNeg )
     _method( LLVMConstNot )
@@ -1636,6 +1772,7 @@ static PyMethodDef core_methods[] = {
 
     /* Global Variables */
     _method( LLVMAddGlobal )
+    _method( LLVMAddGlobalInAddressSpace )
     _method( LLVMGetNamedGlobal )
     _method( LLVMGetFirstGlobal )
     _method( LLVMGetNextGlobal )
@@ -1644,8 +1781,8 @@ static PyMethodDef core_methods[] = {
     _method( LLVMGetInitializer )
     _method( LLVMSetInitializer )
     _method( LLVMSetThreadLocal )
-    _method( LLVMSetGlobalConstant )
     _method( LLVMIsThreadLocal )
+    _method( LLVMSetGlobalConstant )
     _method( LLVMIsGlobalConstant )
 
     /* Functions */
@@ -1666,6 +1803,7 @@ static PyMethodDef core_methods[] = {
     _method( LLVMViewFunctionCFGOnly )
     _method( LLVMAddFunctionAttr )
     _method( LLVMRemoveFunctionAttr )
+    _method( LLVMGetFunctionFromInlineAsm )
 
     /* Arguments */
     _method( LLVMCountParams )
@@ -1687,6 +1825,20 @@ static PyMethodDef core_methods[] = {
     _method( LLVMInsertBasicBlock )
     _method( LLVMDeleteBasicBlock )
 
+    /* MetaData */
+    _method( LLVMMetaDataGet )
+    _method( LLVMGetNamedMetadataOperands )
+    _method( LLVMAddNamedMetadataOperand )
+    _method( LLVMMetaDataGetOperand )
+    _method( LLVMMetaDataGetNumOperands )
+    _method( LLVMMetaDataStringGet )
+
+    /* NamedMetaData */
+    _method( LLVMNamedMetaDataGetName )
+    _method( LLVMNamedMetaDataAddOperand )
+    _method( LLVMEraseNamedMetaData )
+    _method( LLVMDumpNamedMDToString )
+
     /* Instructions */
     _method( LLVMGetInstructionParent )
     _method( LLVMGetFirstInstruction )
@@ -1704,6 +1856,8 @@ static PyMethodDef core_methods[] = {
     _method( LLVMInstGetOpcode )
     _method( LLVMInstGetOpcodeName )
 
+    _method( LLVMInstSetMetaData )
+
     /* Call Sites (Call or Invoke) */
     _method( LLVMSetInstructionCallConv )
     _method( LLVMGetInstructionCallConv )
@@ -1712,6 +1866,8 @@ static PyMethodDef core_methods[] = {
     _method( LLVMAddInstrAttribute )
     _method( LLVMRemoveInstrAttribute )
     _method( LLVMSetInstrParamAlignment )
+    _method( LLVMInstGetCalledFunction )
+    _method( LLVMInstSetCalledFunction )
 
     /* PHI Nodes */
     _method( LLVMAddIncoming1 )
@@ -2033,7 +2189,7 @@ static PyMethodDef core_methods[] = {
     _method( LLVMLoadLibraryPermanently )
     _method( LLVMParseEnvOpts )
 
-    //_method( LLVMInlineFunction )
+    _method( LLVMInlineFunction )
     _method( PyCObjectVoidPtrToPyLong )
     { NULL }
 };
