@@ -296,6 +296,9 @@ def unpack_types(objlst):     return _util.unpack_gen(objlst, check_is_type)
 def unpack_values(objlst):    return _util.unpack_gen(objlst, check_is_value)
 def unpack_constants(objlst): return _util.unpack_gen(objlst, check_is_constant)
 
+def unpack_values_or_none(objlst):
+    return _util.unpack_gen_allow_none(objlst, check_is_value)
+
 def check_is_callable(obj):
     if isinstance(obj, Function):
         return
@@ -340,11 +343,20 @@ class Module(llvm.Ownable, llvm.Cacheable):
         return Module(_core.LLVMModuleCreateWithName(id))
 
     @staticmethod
-    def from_bitcode(fileobj):
+    def from_bitcode(fileobj_or_str):
         """Create a Module instance from the contents of a bitcode
-        file."""
+        file.
 
-        data = fileobj.read()
+        fileobj_or_str -- takes a file-like object or string that contains
+                          a module represented in bitcode.
+        """
+
+        if isinstance(fileobj_or_str, _util.unicode_type):
+            data = fileobj_or_str.encode('utf-8')
+        elif isinstance(fileobj_or_str, bytes):
+            data = fileobj_or_str
+        else:
+            data = fileobj_or_str.read()
         ret = _core.LLVMGetModuleFromBitcode(data)
         if not ret:
             raise llvm.LLVMException("Unable to create module from bitcode")
@@ -354,11 +366,23 @@ class Module(llvm.Ownable, llvm.Cacheable):
             return Module(ret)
 
     @staticmethod
-    def from_assembly(fileobj):
+    def from_assembly(fileobj_or_str):
         """Create a Module instance from the contents of an LLVM
-        assembly (.ll) file."""
+        assembly (.ll) file.
 
-        data = fileobj.read()
+
+        fileobj_or_str -- takes a file-like object or string that contains
+                          a module represented in llvm-ir assembly.
+        """
+
+        if isinstance(fileobj_or_str, _util.unicode_type):
+            data = fileobj_or_str.encode('utf-8')
+        elif isinstance(fileobj_or_str, bytes):
+            data = fileobj_or_str
+        else:
+            data = fileobj_or_str.read()
+            if isinstance(data, _util.unicode_type):
+                data = data.encode()
         ret = _core.LLVMGetModuleFromAssembly(data)
         if not ret:
             raise llvm.LLVMException("Unable to create module from assembly")
@@ -454,9 +478,9 @@ class Module(llvm.Ownable, llvm.Cacheable):
             return _make_type(ptr, kind)
         return None
 
-    def add_global_variable(self, ty, name):
+    def add_global_variable(self, ty, name, addrspace=0):
         """Add a global variable of given type with given name."""
-        return GlobalVariable.new(self, ty, name)
+        return GlobalVariable.new(self, ty, name, addrspace)
 
     def get_global_variable_named(self, name):
         """Return a GlobalVariable object for the given name."""
@@ -496,14 +520,24 @@ class Module(llvm.Ownable, llvm.Cacheable):
         if ret != "":
             raise llvm.LLVMException(ret)
 
-    def to_bitcode(self, fileobj):
+    def to_bitcode(self, fileobj=None):
         """Write bitcode representation of module to given file-like
-        object."""
+        object.
+
+        fileobj -- A file-like object to where the bitcode is written.
+                   If it is None, the bitcode is returned.
+
+        Return value -- Returns None if fileobj is not None.
+                        Otherwise, return the bitcode as a bytestring.
+        """
 
         data = _core.LLVMGetBitcodeFromModule(self.ptr)
         if not data:
             raise llvm.LLVMException("Unable to create bitcode")
-        fileobj.write(data)
+        if fileobj is not None:
+            fileobj.write(data)
+        else:
+            return data
 
     def add_library(self, name):
         return _core.LLVMModuleAddLibrary(self.ptr, name)
@@ -516,15 +550,40 @@ class Module(llvm.Ownable, llvm.Cacheable):
 
     id = property(_get_id, _set_id)
 
-    def to_native_object(self):
-        '''returns byte string of the module as native object code
-        '''
-        return _core.LLVMGetNativeCodeFromModule(self.ptr, 0)
+    def to_native_object(self, fileobj=None):
+        '''Outputs the byte string of the module as native object code
 
-    def to_native_assembly(self):
-        '''returns byte string of the module as native assembly code
+        If a fileobj is given, the output is written to it;
+        Otherwise, the output is returned
         '''
-        return _core.LLVMGetNativeCodeFromModule(self.ptr, 1)
+        data = _core.LLVMGetNativeCodeFromModule(self.ptr, 0)
+        if fileobj is not None:
+            fileobj.write(data)
+        else:
+            return data
+
+    def to_native_assembly(self, fileobj=None):
+        '''Outputs the byte string of the module as native assembly code
+
+        If a fileobj is given, the output is written to it;
+        Otherwise, the output is returned
+        '''
+        data = _core.LLVMGetNativeCodeFromModule(self.ptr, 1)
+        if fileobj is not None:
+            fileobj.write(data)
+        else:
+            return data
+
+    def get_or_insert_named_metadata(self, name):
+        ptr = _core.LLVMModuleGetOrInsertNamedMetaData(self.ptr, name)
+        return NamedMetaData(ptr)
+
+    def get_named_metadata(self, name):
+        ptr = _core.LLVMModuleGetNamedMetaData(self.ptr, name)
+        return NamedMetaData(ptr)
+
+    def clone(self):
+        return Module(_core.LLVMCloneModule(self.ptr))
 
 #===----------------------------------------------------------------------===
 # Types
@@ -1172,7 +1231,13 @@ class Constant(User):
 
 
 class ConstantExpr(Constant):
-    pass
+    @property
+    def opcode(self):
+        return _core.LLVMGetConstExprOpcode(self.ptr)
+
+    @property
+    def opcode_name(self):
+        return _core.LLVMGetConstExprOpcodeName(self.ptr)
 
 
 class ConstantAggregateZero(Constant):
@@ -1187,7 +1252,17 @@ class ConstantDataVector(Constant):
     pass
 
 class ConstantInt(Constant):
-    pass
+    @property
+    def z_ext_value(self):
+        '''Obtain the zero extended value for an integer constant value.'''
+        # Warning: assertion failure when value does not fit in 64 bits
+        return _core.LLVMConstIntGetZExtValue(self.ptr)
+
+    @property
+    def s_ext_value(self):
+        '''Obtain the sign extended value for an integer constant value.'''
+        # Warning: assertion failure when value does not fit in 64 bits
+        return _core.LLVMConstIntGetSExtValue(self.ptr)
 
 
 class ConstantFP(Constant):
@@ -1259,14 +1334,13 @@ class GlobalValue(Constant):
     def module(self):
         return Module(_core.LLVMGetGlobalParent(self.ptr))
 
-
 class GlobalVariable(GlobalValue):
 
     @staticmethod
-    def new(module, ty, name):
+    def new(module, ty, name, addrspace=0):
         check_is_module(module)
         check_is_type(ty)
-        return _make_value(_core.LLVMAddGlobal(module.ptr, ty.ptr, name))
+        return _make_value(_core.LLVMAddGlobalInAddressSpace(module.ptr, ty.ptr, name, addrspace))
 
     @staticmethod
     def get(module, name):
@@ -1292,6 +1366,9 @@ class GlobalVariable(GlobalValue):
         check_is_constant(const)
         _core.LLVMSetInitializer(self.ptr, const.ptr)
 
+    def _del_initializer(self):
+        check_is_constant(const)
+
     initializer = property(_get_initializer, _set_initializer)
 
     def _get_is_global_constant(self):
@@ -1304,6 +1381,13 @@ class GlobalVariable(GlobalValue):
     global_constant = \
         property(_get_is_global_constant, _set_is_global_constant)
 
+    def _get_thread_local(self):
+        return bool(_core.LLVMIsThreadLocal(self.ptr))
+    def _set_thread_local(self, value):
+        value = _to_int(value)
+        _core.LLVMSetThreadLocal(self.ptr, value)
+
+    thread_local = property(_get_thread_local, _set_thread_local)
 
 class Argument(Value):
 
@@ -1421,6 +1505,94 @@ class Function(GlobalValue):
         # failure, it appears to print result to stderr and abort.
         return _core.LLVMVerifyFunction(self.ptr) != 0
 
+#===----------------------------------------------------------------------===
+# InlineAsm
+#===----------------------------------------------------------------------===
+
+class InlineAsm(Value):
+    @staticmethod
+    def get(functype, asm, constrains, side_effect=False,
+                   align_stack=False, dialect=0):
+        assert dialect == 0, "LLVM 3.1 does not implement dialect selection"
+        check_is_type(functype)
+        pycap = _core.LLVMGetFunctionFromInlineAsm(functype.ptr, asm,
+                                                   constrains, int(side_effect),
+                                                   int(align_stack), dialect)
+        return _make_value(pycap)
+
+#===----------------------------------------------------------------------===
+# MetaData
+#===----------------------------------------------------------------------===
+
+class MetaData(Value):
+    @staticmethod
+    def get(module, values):
+        '''
+        values -- must be an iterable of Constant or None. None is treated as "null".
+        '''
+        vs = unpack_values_or_none(values)
+        ptr = _core.LLVMMetaDataGet(module.ptr, vs)
+        return MetaData(ptr)
+
+    @staticmethod
+    def get_named_operands(module, name):
+        lst = _core.LLVMGetNamedMetadataOperands(module.ptr, name)
+        return [MetaData(ptr) for ptr in lst]
+
+    @staticmethod
+    def add_named_operand(module, name, operand):
+        _core.LLVMAddNamedMetadataOperand(module.ptr, name, operand.ptr)
+
+    @property
+    def operand_count(self):
+        return _core.LLVMMetaDataGetNumOperands(self.ptr)
+
+    @property
+    def operands(self):
+        """Yields operands of this metadata."""
+        return [self._get_operand(i) for i in range(self.operand_count)]
+
+    def _get_operand(self, i):
+        val = _core.LLVMMetaDataGetOperand(self.ptr, i)
+        if val: # metadata operands can be None
+            return _make_value(val)
+
+class MetaDataString(Value):
+    @staticmethod
+    def get(module, s):
+        ptr = _core.LLVMMetaDataStringGet(module.ptr, s)
+        return MetaDataString(ptr)
+
+    @property
+    def string(self):
+        '''Same as MDString::getString'''
+        return self.name
+
+class NamedMetaData(llvm.Cacheable):
+    @staticmethod
+    def get_or_insert(mod, name):
+        return mod.get_or_insert_named_metadata(name)
+
+    @staticmethod
+    def get(mod, name):
+        return mod.get_named_metadata(name)
+
+    def __init__(self, ptr):
+        self.ptr = ptr
+
+    def delete(self):
+        _core.LLVMEraseNamedMetaData(self.ptr)
+
+    @property
+    def name(self):
+        return _core.LLVMNamedMetaDataGetName(self.ptr)
+
+    def __str__(self):
+        return _core.LLVMDumpNamedMDToString(self.ptr)
+
+    def add(self, operand):
+        _core.LLVMNamedMetaDataAddOperand(self.ptr, operand.ptr)
+
 #===----------------------------------------------------------------------===
 # Instruction
 #===----------------------------------------------------------------------===
@@ -1471,6 +1643,9 @@ class Instruction(User):
     def set_volatile(self, flag):
         return _core.LLVMSetVolatile(self.ptr, int(bool(flag)))
 
+    def set_metadata(self, kind, metadata):
+        return _core.LLVMInstSetMetaData(self.ptr, kind, metadata.ptr)
+
     @property
     def opcode(self):
         return _core.LLVMInstGetOpcode(self.ptr)
@@ -1495,6 +1670,16 @@ class CallOrInvokeInstruction(Instruction):
     def set_parameter_alignment(self, idx, align):
         _core.LLVMSetInstrParamAlignment(self.ptr, idx, align)
 
+    def _get_called_function(self):
+        function = _core.LLVMInstGetCalledFunction(self.ptr)
+        if function: # Return value can be None on indirect call/invoke
+            return _make_value(function)
+
+    def _set_called_function(self, function):
+        _core.LLVMInstSetCalledFunction(self.ptr, function.ptr)
+
+    called_function = property(_get_called_function, _set_called_function)
+
     # tail call is valid only for 'call', not 'invoke'
     # disabled for now
     #def _get_tc(self): return _core.LLVMIsTailCall(self.ptr)
@@ -1582,6 +1767,9 @@ __class_for_valueid = {
     VALUE_CONSTANT_STRUCT                 : ConstantStruct,
     VALUE_CONSTANT_VECTOR                 : ConstantVector,
     VALUE_CONSTANT_POINTER_NULL           : ConstantPointerNull,
+    VALUE_MD_NODE                         : MetaData,
+    VALUE_MD_STRING                       : MetaDataString,
+    VALUE_INLINE_ASM                      : InlineAsm,
     VALUE_INSTRUCTION + OPCODE_PHI        : PHINode,
     VALUE_INSTRUCTION + OPCODE_CALL       : CallOrInvokeInstruction,
     VALUE_INSTRUCTION + OPCODE_INVOKE     : CallOrInvokeInstruction,
@@ -1840,13 +2028,17 @@ class Builder(object):
         check_is_value(ptr)
         return _make_value(_core.LLVMBuildFree(self.ptr, ptr.ptr))
 
-    def load(self, ptr, name="", align=0, volatile=False):
+    def load(self, ptr, name="", align=0, volatile=False, invariant=False):
         check_is_value(ptr)
         inst = _make_value(_core.LLVMBuildLoad(self.ptr, ptr.ptr, name))
         if align:
             _core.LLVMLdSetAlignment(inst.ptr, align)
         if volatile:
             inst.set_volatile(volatile)
+        if invariant:
+            mod = self.basic_block.function.module
+            md = MetaData.get(mod, []) # empty metadata node
+            inst.set_metadata('invariant.load', md)
         return inst
 
     def store(self, value, ptr, align=0, volatile=False):
@@ -1976,6 +2168,11 @@ class Builder(object):
 
     def call(self, fn, args, name=""):
         check_is_callable(fn)
+
+        err_template = 'Argument type mismatch: expected %s but got %s'
+        for i, (t, v) in enumerate(zip(fn.type.pointee.args, args)):
+            if  t != v.type:
+                raise TypeError(err_template % (t, v.type))
         arg_ptrs = unpack_values(args)
         return _make_value(
             _core.LLVMBuildCall(self.ptr, fn.ptr, arg_ptrs, name))
@@ -2145,7 +2342,7 @@ def load_library_permanently(filename):
 
 def inline_function(call):
     check_is_value(call)
-    return _core.LLVMInlineFunction(call.ptr)
+    return bool(_core.LLVMInlineFunction(call.ptr))
 
 def parse_environment_options(progname, envname):
     _core.LLVMParseEnvOpts(progname, envname)
