@@ -74,6 +74,8 @@
 #include "llvm/Transforms/Utils/Cloning.h"
 #include "llvm/Linker.h"
 #include "llvm/Support/SourceMgr.h"
+#include <llvm/InlineAsm.h>
+
 
 // LLVM-C includes
 #include "llvm-c/Core.h"
@@ -83,11 +85,19 @@
 #include "extra.h"
 #include "llvm_c_extra.h"
 
-
 namespace llvm{
 DEFINE_SIMPLE_CONVERSION_FUNCTIONS(EngineBuilder, LLVMEngineBuilderRef)
 DEFINE_SIMPLE_CONVERSION_FUNCTIONS(TargetMachine, LLVMTargetMachineRef)
+DEFINE_SIMPLE_CONVERSION_FUNCTIONS(NamedMDNode, LLVMNamedMDRef)
+
+template<typename T>
+inline T **unwrap(LLVMTypeRef *Tys, unsigned Length) {
+    (void)Length;
+    return reinterpret_cast<T**>(Tys);
 }
+
+}
+
 /*
  * For use in LLVMDumpPasses to dump passes.
  */
@@ -126,6 +136,112 @@ const CodeGenOpt::Level OptLevelMap[] = {
 };
 } // end anony namespace
 
+
+LLVMValueRef LLVMGetFunctionFromInlineAsm(LLVMTypeRef funcType,
+                                          const char inlineAsm[],
+                                          const char constrains[],
+                                          bool hasSideEffect,
+                                          bool isAlignStack,
+                                          int asmDialect)
+{
+    using namespace llvm;
+    FunctionType *fnty = unwrap<FunctionType>(funcType);
+    // asmDialect does not exist for LLVM 3.1
+    InlineAsm *inlineasmobj = InlineAsm::get(fnty, inlineAsm, constrains,
+                                             hasSideEffect, isAlignStack);
+    return wrap(inlineasmobj);
+}
+
+LLVMModuleRef LLVMCloneModule(LLVMModuleRef mod)
+{
+    using namespace llvm;
+    return wrap(CloneModule(unwrap(mod)));
+}
+
+const char * LLVMDumpNamedMDToString(LLVMNamedMDRef nmd)
+{
+    using namespace llvm;
+    std::string s;
+    llvm::raw_string_ostream buf(s);
+    unwrap(nmd)->print(buf, NULL);
+    return strdup(buf.str().c_str());
+}
+
+const char * LLVMNamedMetaDataGetName(LLVMNamedMDRef nmd)
+{
+    using namespace llvm;
+    return unwrap(nmd)->getName().data();
+}
+
+void LLVMNamedMetaDataAddOperand(LLVMNamedMDRef nmd, LLVMValueRef md)
+{
+    using namespace llvm;
+    unwrap(nmd)->addOperand(unwrap<MDNode>(md));
+}
+
+void LLVMEraseNamedMetaData(LLVMNamedMDRef nmd)
+{
+    using namespace llvm;
+    unwrap(nmd)->eraseFromParent();
+}
+
+LLVMNamedMDRef LLVMModuleGetOrInsertNamedMetaData(LLVMModuleRef mod, const char *name)
+{
+    using namespace llvm;
+    return wrap(unwrap(mod)->getOrInsertNamedMetadata(name));
+}
+
+LLVMNamedMDRef LLVMModuleGetNamedMetaData(LLVMModuleRef mod, const char *name)
+{
+    using namespace llvm;
+    return wrap(unwrap(mod)->getNamedMetadata(name));
+}
+
+void LLVMInstSetMetaData(LLVMValueRef instref, const char* mdkind,
+                         LLVMValueRef metaref)
+{
+    using namespace llvm;
+    unwrap<Instruction>(instref)->setMetadata(mdkind, unwrap<MDNode>(metaref));
+}
+
+LLVMValueRef LLVMMetaDataGet(LLVMModuleRef modref, LLVMValueRef * valrefs,
+                             unsigned valct)
+{
+    using namespace llvm;
+    LLVMContext & context = unwrap(modref)->getContext();
+    MDNode * const node = MDNode::get(
+                            context,
+                            makeArrayRef(unwrap<Value>(valrefs, valct), valct));
+    return wrap(node);
+}
+
+LLVMValueRef LLVMMetaDataGetOperand(LLVMValueRef mdref, unsigned index)
+{
+    return wrap(llvm::unwrap<llvm::MDNode>(mdref)->getOperand(index));
+}
+
+unsigned LLVMMetaDataGetNumOperands(LLVMValueRef mdref)
+{
+    return llvm::unwrap<llvm::MDNode>(mdref)->getNumOperands();
+}
+
+LLVMValueRef LLVMMetaDataStringGet(LLVMModuleRef modref, const char *s)
+{
+    LLVMContext & context = unwrap(modref)->getContext();
+    MDString * const mdstring = MDString::get(context, s);
+    return wrap(mdstring);
+}
+
+const char *LLVMGetConstExprOpcodeName(LLVMValueRef inst)
+{
+    return llvm::unwrap<llvm::ConstantExpr>(inst)->getOpcodeName();
+}
+
+unsigned LLVMGetConstExprOpcode(LLVMValueRef inst)
+{
+    return llvm::unwrap<llvm::ConstantExpr>(inst)->getOpcode();
+}
+
 void LLVMLdSetAlignment(LLVMValueRef inst, unsigned align)
 {
     return llvm::unwrap<LoadInst>(inst)->setAlignment(align);
@@ -221,7 +337,7 @@ void LLVMDisposeTargetMachine(LLVMTargetMachineRef tm){
 
 unsigned char* LLVMTargetMachineEmitFile(LLVMTargetMachineRef tmref,
                                          LLVMModuleRef modref,
-                                         int assembly, unsigned * lenp,
+                                         int assembly, size_t * lenp,
                                          std::string &error)
 {
     using namespace llvm;
@@ -270,10 +386,6 @@ unsigned char* LLVMTargetMachineEmitFile(LLVMTargetMachineRef tmref,
     // and then into a new buffer
     size_t bclen = bc.size();
     unsigned char *bytes = new unsigned char[bclen];
-    if (!bytes){
-        error = "Out of memory";
-        return NULL;
-    }
     memcpy(bytes, bc.data(), bclen);
 
     /* return */
@@ -319,7 +431,7 @@ LLVMTargetDataRef LLVMTargetMachineGetTargetData(LLVMTargetMachineRef tm)
 }
 
 unsigned char* LLVMGetNativeCodeFromModule(LLVMModuleRef module, int assembly,
-                                           unsigned * lenp, std::string &error)
+                                           size_t * lenp, std::string &error)
 {
     using namespace llvm;
     assert(lenp);
@@ -737,47 +849,59 @@ unsigned LLVMCmpInstGetPredicate(LLVMValueRef cmpinst)
     return instp->getPredicate();
 }
 
-/* llvm::unwrap a set of `n' wrapped objects starting at `values',
- * into a vector of pointers to llvm::unwrapped objects `out'. */
-template <typename W, typename UW>
-void unwrap_vec(W *values, unsigned n, std::vector<UW *>& out)
+LLVMValueRef LLVMInstGetCalledFunction(LLVMValueRef inst)
 {
-    out.clear();
-    out.reserve(n);
-    while (n--) {
-        UW *p = llvm::unwrap(*values);
-        assert(p);
-        out.push_back(p);
-        ++values;
-    }
+    llvm::Instruction *instp = llvm::unwrap<llvm::Instruction>(inst);
+    return llvm::wrap(CallSite(instp).getCalledFunction());
 }
 
-/* Same as llvm::unwrap_vec, but use a vector of const pointers. */
-template <typename W, typename UW>
-void unwrap_cvec(W *values, unsigned n, std::vector<const UW *>& out)
+void LLVMInstSetCalledFunction(LLVMValueRef inst, LLVMValueRef fn)
 {
-    out.clear();
-
-    while (n--) {
-        UW *p = llvm::unwrap(*values);
-        assert(p);
-        out.push_back(p);
-        ++values;
-    }
+    using namespace llvm;
+    Instruction *instp = unwrap<Instruction>(inst);
+    CallSite(instp).setCalledFunction(unwrap(fn));
 }
 
+///* llvm::unwrap a set of `n' wrapped objects starting at `values',
+// * into a vector of pointers to llvm::unwrapped objects `out'. */
+//template <typename W, typename UW>
+//void unwrap_vec(W *values, unsigned n, std::vector<UW *>& out)
+//{
+//    out.clear();
+//    out.reserve(n);
+//    while (n--) {
+//        UW *p = llvm::unwrap(*values);
+//        assert(p);
+//        out.push_back(p);
+//        ++values;
+//    }
+//}
+
+///* Same as llvm::unwrap_vec, but use a vector of const pointers. */
+//template <typename W, typename UW>
+//void unwrap_cvec(W *values, unsigned n, std::vector<const UW *>& out)
+//{
+//    out.clear();
+
+//    while (n--) {
+//        UW *p = llvm::unwrap(*values);
+//        assert(p);
+//        out.push_back(p);
+//        ++values;
+//    }
+//}
+
+
 LLVMValueRef LLVMBuildRetMultiple(LLVMBuilderRef builder,
     LLVMValueRef *values, unsigned n_values)
 {
+    using namespace llvm;
     assert(values);
 
-    std::vector<llvm::Value *> values_vec;
-    unwrap_vec(values, n_values, values_vec);
-
-    llvm::IRBuilder<> *builderp = llvm::unwrap(builder);
+    IRBuilder<> *builderp = unwrap(builder);
     assert(builderp);
 
-    return llvm::wrap(builderp->CreateAggregateRet(&values_vec[0], values_vec.size()));
+    return wrap(builderp->CreateAggregateRet(unwrap<Value>(values, n_values), n_values));
 }
 
 LLVMValueRef LLVMBuildGetResult(LLVMBuilderRef builder,
@@ -819,7 +943,7 @@ unsigned LLVMValueGetUses(LLVMValueRef value, LLVMValueRef **refs)
         return 0;
 
     assert(refs);
-    LLVMValueRef *out = (LLVMValueRef *)malloc(sizeof(LLVMValueRef) * n);
+    LLVMValueRef *out = new LLVMValueRef[n];
     if (!out)
         return 0;
     *refs = out;
@@ -837,7 +961,7 @@ unsigned LLVMValueGetUses(LLVMValueRef value, LLVMValueRef **refs)
 void LLVMDisposeValueRefArray(LLVMValueRef *refs)
 {
     assert(refs);
-    free(refs);
+    delete [] refs;
 }
 
 unsigned LLVMUserGetNumOperands(LLVMValueRef user)
@@ -875,16 +999,15 @@ void LLVMSetDoesNotThrow(LLVMValueRef fn, int DoesNotThrow)
 LLVMValueRef LLVMGetIntrinsic(LLVMModuleRef module, int id,
     LLVMTypeRef *types, unsigned n_types)
 {
+    using namespace llvm;
     assert(types);
 
-    std::vector< llvm::Type* > types_vec;
-    unwrap_vec(types, n_types, types_vec);
-
-    llvm::Module *modulep = llvm::unwrap(module);
+    Module *modulep = unwrap(module);
     assert(modulep);
 
-    llvm::Function *intfunc = llvm::Intrinsic::getDeclaration(modulep,
-				     llvm::Intrinsic::ID(id), types_vec);
+    Function *intfunc = Intrinsic::getDeclaration(modulep, Intrinsic::ID(id),
+                                    makeArrayRef(unwrap<Type>(types, n_types),
+                                                 n_types));
 
     return wrap(intfunc);
 }
@@ -949,7 +1072,7 @@ unsigned LLVMLinkModules(LLVMModuleRef dest, LLVMModuleRef src, unsigned int mod
 }
 #endif
 
-unsigned char *LLVMGetBitcodeFromModule(LLVMModuleRef module, unsigned *lenp)
+unsigned char *LLVMGetBitcodeFromModule(LLVMModuleRef module, size_t *lenp)
 {
     assert(lenp);
 
@@ -962,11 +1085,9 @@ unsigned char *LLVMGetBitcodeFromModule(LLVMModuleRef module, unsigned *lenp)
     llvm::WriteBitcodeToFile(modulep, buf);
     const std::string& bc = buf.str();
 
-    /* and then into a malloc()-ed block */
+    /* and then into a new()-ed block */
     size_t bclen = bc.size();
-    unsigned char *bytes = (unsigned char *)malloc(bclen);
-    if (!bytes)
-        return NULL;
+    unsigned char *bytes = new unsigned char[bclen];
     memcpy(bytes, bc.data(), bclen);
 
     /* return */
@@ -1013,16 +1134,15 @@ int LLVMInlineFunction(LLVMValueRef call)
     llvm::Value *callp = llvm::unwrap(call);
     assert(callp);
 
-//    llvm::CallSite cs = llvm::CallSite::get(callp);
-    llvm::CallSite cs;
+    llvm::InlineFunctionInfo unused;
+
     llvm::Instruction *II = llvm::dyn_cast<llvm::Instruction>(callp);
     if (II->getOpcode() == llvm::Instruction::Call)
-        cs = llvm::CallSite(static_cast<llvm::CallInst*>(II));
+        return llvm::InlineFunction(static_cast<llvm::CallInst*>(II), unused);
     else if (II->getOpcode() == llvm::Instruction::Invoke)
-        cs = llvm::CallSite(static_cast<llvm::InvokeInst*>(II));
-
-    llvm::InlineFunctionInfo unused;
-    return llvm::InlineFunction(cs, unused);
+        return llvm::InlineFunction(static_cast<llvm::InvokeInst*>(II), unused);
+    else
+        return 0;
 }
 
 unsigned LLVMGetParamAlignment(LLVMValueRef arg)
