@@ -47,12 +47,96 @@
 
 #include "llvm-c/Transforms/PassManagerBuilder.h"
 
+
 #include "llvm_c_extra.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*
+ * Wraps llvm:InlineAsm::get
+ */
+LLVMValueRef LLVMGetFunctionFromInlineAsm(LLVMTypeRef funcType,
+                                          const char inlineAsm[],
+                                          const char constrains[],
+                                          bool hasSideEffect,
+                                          bool isAlignStack,
+                                          int asmDialect);
+
+/*
+ * Wraps llvm::CloneModule
+ */
+LLVMModuleRef LLVMCloneModule(LLVMModuleRef mod);
+
+
+/*
+ * Wraps NamedMDNode::print()
+ */
+const char * LLVMDumpNamedMDToString(LLVMNamedMDRef nmd);
+
+/*
+ * Wraps NamedMDNode::getName()
+ */
+const char * LLVMNamedMetaDataGetName(LLVMNamedMDRef nmd);
+
+/*
+ * Wraps NamedMDNode::addOperand()
+ */
+void LLVMNamedMetaDataAddOperand(LLVMNamedMDRef nmd, LLVMValueRef md);
+
+/*
+ * Wraps NamedMDNode::eraseFromParent()
+ */
+void LLVMEraseNamedMetaData(LLVMNamedMDRef nmd);
+
+/*
+ * Wraps Module::getOrInsertNamedMetadata
+ */
+LLVMNamedMDRef LLVMModuleGetOrInsertNamedMetaData(LLVMModuleRef mod, const char *name);
+
+/*
+ * Wraps Module::getNamedMetadata
+ */
+LLVMNamedMDRef LLVMModuleGetNamedMetaData(LLVMModuleRef mod, const char *name);
+
+/*
+ * Wraps Instruction::setMetadata()
+ */
+void LLVMInstSetMetaData(LLVMValueRef instref, const char* mdkind,
+                         LLVMValueRef metaref);
+
+/*
+ * Wraps MDNode::get()
+ */
+LLVMValueRef LLVMMetaDataGet(LLVMModuleRef modref, LLVMValueRef * valrefs,
+                             unsigned valct);
+
+/*
+ * Wraps MDNode::getOperand()
+ */
+LLVMValueRef LLVMMetaDataGetOperand(LLVMValueRef mdref, unsigned index);
+
+/*
+ * Wraps MDNode::getNumOperands()
+ */
+unsigned LLVMMetaDataGetNumOperands(LLVMValueRef mdref);
+
+/*
+ * Wraps MDString::get()
+ */
+LLVMValueRef LLVMMetaDataStringGet(LLVMModuleRef modref, const char *s);
+
+/*
+ * Wraps ConstantExpr::getOpcodeName()
+ */
+const char *LLVMGetConstExprOpcodeName(LLVMValueRef inst);
+
+/*
+ * Wraps ConstantExpr::getOpcode()
+ */
+unsigned LLVMGetConstExprOpcode(LLVMValueRef inst);
+
 /*
  * Wraps LoadInst::SetAlignment
  */
@@ -87,7 +171,7 @@ void LLVMDisposeTargetMachine(LLVMTargetMachineRef tm);
  */
 unsigned char* LLVMTargetMachineEmitFile(LLVMTargetMachineRef tmref,
                                          LLVMModuleRef modref,
-                                         int assembly, unsigned * lenp,
+                                         int assembly, size_t * lenp,
                                          std::string &error);
 
 /*
@@ -129,7 +213,7 @@ void LLVMPrintRegisteredTargetsForVersion();
  * Wraps TargetMachine::addPassesToEmitFile
  */
 unsigned char* LLVMGetNativeCodeFromModule(LLVMModuleRef module, int assembly,
-                                           unsigned * lenp, std::string &error);
+                                           size_t * lenp, std::string &error);
 
 /*
  * Wraps IRBuilder::CreateFence
@@ -400,6 +484,14 @@ unsigned LLVMInstGetOpcode(LLVMValueRef inst);
 /* Wraps llvm::CmpInst::getPredicate(). */
 unsigned LLVMCmpInstGetPredicate(LLVMValueRef cmpinst);
 
+/* Wraps llvm::CallSite::getCalledFunction.
+ */
+LLVMValueRef LLVMInstGetCalledFunction(LLVMValueRef inst);
+
+/* Wraps llvm::CallSite::setCalledFunction.
+ */
+void LLVMInstSetCalledFunction(LLVMValueRef inst, LLVMValueRef fn);
+
 /* Wraps llvm::ParseAssemblyString(). Returns a module reference or NULL (with
  * `out' pointing to an error message). Dispose error message after use, via
  * LLVMDisposeMessage(). */
@@ -420,7 +512,7 @@ unsigned LLVMLinkModules(LLVMModuleRef dest, LLVMModuleRef src,
 #endif
 /* Returns pointer to a heap-allocated block of `*len' bytes containing bit code
  * for the given module. NULL on error. */
-unsigned char *LLVMGetBitcodeFromModule(LLVMModuleRef module, unsigned *len);
+unsigned char *LLVMGetBitcodeFromModule(LLVMModuleRef module, size_t *len);
 
 /* Wraps llvm::sys::DynamicLibrary::LoadLibraryPermanently(). Returns 0 on
  * failure (with errmsg filled in) and 1 on success. Dispose error message after
@@ -433,8 +525,9 @@ void *LLVMGetPointerToFunction(LLVMExecutionEngineRef ee, LLVMValueRef fn);
 
 /* Wraps llvm::InlineFunction(). Inlines a function. C is the call
  * instruction, created by LLVMBuildCall. Even if it fails, the Function
- * containing the call is still in a proper state (not changed). */
-//int LLVMInlineFunction(LLVMValueRef call);
+ * containing the call is still in a proper state (not changed).
+ */
+int LLVMInlineFunction(LLVMValueRef call);
 
 /* Wraps llvm::getAlignmentFromAttrs from Attributes.h. Compliments the
  * already available LLVMSetParamAlignment(). */
